//obsługa wyświetlacza LCD
#include <LiquidCrystal.h>

//port PB
#define pb0  8
#define pb1  9
#define pb2  10
#define pb3  11
#define pb4  12
#define pb5  13

//port D
#define pd0  0
#define pd1  1
#define pd2  2
#define pd3  3
#define pd4  4
#define pd5  5
#define pd6  6
#define pd7  7


//definicje wyprowadzeń dla diod LED
#define  red  11
#define  green  10
#define  blue  12

//liczba linii menu, uwaga: numeracja od 0!
#define  liczba_ekranow_menu  4

//zmienna z nastawą
int liczba_impulsow, okres, czas_trwania_impulsow;
//stan impulsatora
int tmp, y_markera = 0;
//bit ze stanem przycisku i ze stanem enkodera
boolean imp_przycisk, w_lewo, w_prawo, poprzedni;

//dołączenie wyświetlacza
LiquidCrystal lcd(pb0, pb1, pd4, pd5, pd6, pd7);



/**************************************************
 akcja po wyborze pozycji z menu
 **************************************************/
//wywołanie odpowieniej akcji
void akcja(int numer_linii)
{
  czekaj_na_przycisk();
  switch(numer_linii)
  {
    case 0:
//akcja 1 - kolor czerwony
      digitalWrite(green, LOW);
      digitalWrite(blue, LOW);
      digitalWrite(red, HIGH);
      break;
    case 1:
//akcja 2 - kolor zielony
      digitalWrite(red, LOW);
      digitalWrite(blue, LOW);
      digitalWrite(green, HIGH);
      break;
    case 2:
//akcja 3 - kolor niebieski
      digitalWrite(red, LOW);
      digitalWrite(green, LOW);
      digitalWrite(blue, HIGH);
      break;
    case 3:
//akcja 4 - kolor biały
      digitalWrite(red, HIGH);
      digitalWrite(green, HIGH);
      digitalWrite(blue, HIGH);
      break;
    case 4:
    default:
//akcja 5 - wyłączenie LED
      digitalWrite(red, LOW);
      digitalWrite(green, LOW);
      digitalWrite(blue, LOW);
  }
}

/**************************************************
 * obsługa enkodera
 **************************************************/
void odczyt_impulsatora()
{
  boolean imp_pin1, imp_pin2, biezacy;
  unsigned int i = 0;
  //odczyt stanu przycisku enkodera
  imp_przycisk = w_prawo = w_lewo = LOW;
  imp_pin1 = digitalRead(pd0);
  if(imp_pin1 == LOW)
  {
    delay(50);
    imp_pin1 = digitalRead(pd0);
    if(imp_pin1 == LOW) imp_przycisk = HIGH;
    else imp_przycisk = LOW;
  }
  //odczyt kierunku obrotu
  imp_pin1 = digitalRead(pd1);
  if(imp_pin1 == LOW) biezacy = HIGH; 
  else biezacy = LOW;
  if(poprzedni != biezacy)
  {
    poprzedni = biezacy;
    if(imp_pin1 == LOW)
    {
      imp_pin2 = digitalRead(pd2);
      if(imp_pin2 == LOW)
      {
        w_lewo = HIGH;
        w_prawo = LOW;
      }
      else
      {
        w_lewo = LOW;
        w_prawo = HIGH;
      }
    }
  }
}

//oczekiwanie na zwolnienie ośki
void czekaj_na_przycisk()
{
  while (imp_przycisk == HIGH) odczyt_impulsatora();
}


/**************************************************
 * menu główne
 **************************************************/
void menu_glowne(int numer_linii)
{
  switch (numer_linii)
  {
  case 0:
    lcd.setCursor(0, 0);
    lcd.print(">LED    ");
    lcd.setCursor(0, 1);
    lcd.print(" CZERWON");
    break;      
  case 1:
    lcd.setCursor(0, 0);
    lcd.print(">LED    ");
    lcd.setCursor(0, 1);
    lcd.print(" ZIELONY");
    break;      
  case 2:
    lcd.setCursor(0, 0);
    lcd.print(">LED    ");
    lcd.setCursor(0, 1);
    lcd.print(" NIEBIES");
    break;      
  case 3:
    lcd.setCursor(0, 0);
    lcd.print(">LED    ");
    lcd.setCursor(0, 1);
    lcd.print(" BIALY  ");
    break;      
  case 4:
    lcd.setCursor(0, 0);
    lcd.print(">WYLACZ ");
    lcd.setCursor(0, 1);
    lcd.print(" LED    ");
    break;      
  }
}

/**************************************************
 * inicjalizacja mikrokontrolera
 **************************************************/
void setup()
{
  //wejścia enkodera
  pinMode(pd0, INPUT_PULLUP);
  pinMode(pd1, INPUT);
  pinMode(pd2, INPUT);
  //wyprowadzenia sterujące diodą LED
  pinMode(red, OUTPUT);
  pinMode(green, OUTPUT);
  pinMode(blue, OUTPUT);
  //wyświetlenie ekranu początkowego
  lcd.begin(8, 2);
  menu_glowne(0);
}


/**************************************************
 * główna pętla programu
 **************************************************/
void loop()
{
  //odczyt stanu impulsatora
  odczyt_impulsatora();
  //ruch markera w dół
  if (w_lewo == HIGH)
  {
    y_markera--;
    if (y_markera < 0) y_markera = liczba_ekranow_menu;
    //wyświetlenie menu głównego
    menu_glowne(y_markera);
  } 
  else
    //ruch markera w górę
    if (w_prawo == HIGH)
    {
      y_markera++;
      if (y_markera > liczba_ekranow_menu) y_markera = 0;
      //wyświetlenie menu głównego
      menu_glowne(y_markera);
    } 
    else
      //wywołanie akcji po naciśnięciu impulsatora
      //akcja zależna od położenia w menu
      if (imp_przycisk == HIGH)
      {
        akcja(y_markera);
        //przywrócenie menu głównego
        menu_glowne(y_markera);
      }
}

